import warnings
# THIS FILE HAS BEEN ABANDONED
# IT WILL BE REMOVED EVENTUALLY...


# from Eqn_solver.text_parsing import collect_variables
# Fix collect variables reference

# Define the expression whose roots we want to find
# take in an equation that needs to be reordered

"""
incorporate
Equation Object={...}
dictionary of all things

(from Solver.py)
exelist:
['A=2', 'B=5', 'C=4']
results list:
['A', 'B', 'C']

"""


def solve(equation, equation_variables, known_variables):
    '''
    :parameter
    equation
        string
    equation_variables
        list of strings
    known_variables
        list of strings
        This can be all variables or a small sub set of variables to compare
    :returns
    single_equation_run
        executable code to solve a single equation
        ['def equation(c): return (a+b+c - 2)', 'c = fsolve(equation, 1)', 'c=c[0]']
    [str(result_variable)]
        variable for results printing
    '''

    # Added to make the runtime warning exception work
    # changes warnings from scipy to exceptions
    warnings.filterwarnings(action="error", module="scipy")

    # puts equation on one side of the equals
    equation = reorder(equation)

    # separates known variables from variables in equation
    # this should be one variable
    result_variable = set(equation_variables).difference(known_variables)

    # this should raise an error if it's not going to work
    if len(result_variable) is not 1:
        raise Exception("FSOLVE: an Equation Variable Match Error has Occured")
    single_equation_run = func(equation, result_variable)
    return single_equation_run, [str(result_variable)]


def reorder(equation):
    import re
    left, right = re.split(r"[=]", equation)
    reordered = [left, '-', '(', right, ')']
    reordered = ''.join(reordered)
    return reordered


def func(equation, solved_for_variable):
    initial_guess = 1
    v = str(list(solved_for_variable)[0])
    solver_list = [
        "" + v + " = fsolve(",
        "equation, ",
        str(initial_guess), ")"
    ]
    equation_list = [
        "def equation(", v,"):",
        " return (", str(equation),
        ")"
    ]
    formating = [
        v + "=" + v + "[0]"
    ]
    list1 = [''.join(equation_list), ''.join(solver_list), ''.join(formating)]
    return list1


def DivZeroCheck():
    from scipy.optimize import fsolve

    # 0= equation
    def equation(c):
        return 2 * c + 1
    try:
        ans = equation(0)
    except ZeroDivisionError:
        print("Divide by zero error")
    else:
        c = fsolve(equation, 2)
        c = c[0]
        print('C = ', c)
    return True

if __name__ == "__main__":
    import pprint
    # included fsolve import statement by default
    exec('from scipy.optimize import fsolve')
    singeq = solve("x+y=2+y", ["y", "x"], ["x", "z"])[0]
    singeq2 = solve("a+b+c=2", ["a", "b", "c"], ["a", "b", "d", "e"])[0]
    a = 2
    b = 2
    x = 3
    z = 5

    pprint.pprint(singeq2, indent=1, width=60)
    singeq = ['def equation(y): return (y)',
              'y = fsolve(equation, 1)',
              'y=y[0]']
    pprint.pprint(singeq, indent=1, width=60)
    try:
        for i in singeq+singeq2:
            exec(i)
        print("\nY = " + str(y))
        print("\nX = " + str(x))
        print("\nC = " + str(c))
    except RuntimeWarning:
        # add a thing to happen when this doesn't work
        print("\nTOO MANY COOKS: Not making progress on this equation")
        print("Possible issues:" +
              "\n- May be out of bounds for solver"
              "\n- May be inappropriate guess value"
              "\n- May result in (+/-) infinity"
              "\n- May be division by zero error")
    except NameError:
        print("NameError: solver not fully defined")
    print("\n Divzero check")
    DivZeroCheck()


#!/usr/bin/python3
"""
Module Docstring

THEENDS (The Ends)
THermodynamic Engineering Equations NeeD Solving

1. Read in file of equations
    readfile.py
2. Create Equations Object from text
    EquationObject.py
3. Check Equations for errors
    EquationObject.py --> EquationErrorCheck.py
4. Run Equations through Solver/Parser
    Solver.py
5. run and export results
    RunAndOutput.py

"""
import time
import sys
from the_ends.readfile import readfile
from the_ends.EquationObject import EquationsClass
from the_ends.Solver import Solver
from the_ends.RunAndOutput import solve_and_print_results as results


def main(args=None, debug=False):
    """ Main entry point of the app 
    add: equations = EquationsClass(rf.readfile("1eqn")
    
    
    """
    if args is None:
        args = sys.argv[1]
    else:
        print("something is wrong with how main is being called")
    input_filename = args
    input_string = readfile(input_filename)
    start_time = time.time()
    user_input = EquationsClass(input_string)
    solve = user_input.check()

    if solve == 1:
        exelist, resultslist = Solver(user_input.equations, debug).solve()
        resultsout = results(user_input.entered_equations,exelist, resultslist)
        # print('**************************************************')
        # print('Number of Equations = ', len(user_input.equation_dict()))
        # print('Number of Variables = ', len(user_input.variables()))
        # print('List of Variables', ', '.join(user_input.variables()))
        print("Time Elapsed: {:.3f}s".format(time.time() - start_time))
        # print('**************************************************')
    else:
        resultsout = "Unsolvable"
    print(resultsout)
    return resultsout


if __name__ == "__main__":
    main()


class EquationsClass(object):
    """docstring
    Inputs
    -----------------------------
        eqn_input: text file as a string input into
                   solver for initial processing

    Availible outputs:
    -----------------------------
        self.equation_dict = {"equation":[LeftSide, RightSide]
        self.format
        self.variables
        self.solve
        self.check
        self.parse_eqns_from_string
        self.debug
    """

    def __init__(self, eqn_input):
        """Constructor"""

        if isinstance(eqn_input, str):
            self.equations = self.parse_eqns_from_string(instring=eqn_input)
            self.entered_equations = self.equations
            self.equations = self.format(self.equations)
        elif isinstance(eqn_input, list):
            self.entered_equations = eqn_input
            self.equations = eqn_input
            self.equations = self.format(self.equations)
        else:
            print(str(eqn_input) + "\n has generated an error going into the EquationsClass")
        self.variable_dictionary = self.variable_dict()

    def solve(self):
        from Eqn_solver.Solver import Solver
        solver_object = Solver(self.equations)

        return

    def check(self):
        """
        :return:
            boolean for passing syntax check

        """
        solve = True  # indication for no errors in preprocessing
        containsoperator = False
        count = 0
        tmp_eqn = []
        output_equations = []
        symbols_not_allowed = ['?', '@', '&', '`', '~', '#', '!', '$', '%']

        for text in self.equations:
            # string containing an equation
            # checks every equation line
            count += 1
            containsoperator = False
            for i in text:
                # checks for individual characters in the equations
                # from symbols_not_allowed variable list
                if i in symbols_not_allowed:
                    print("Error in equation " + str(count)
                          + ": \"" + i + "\" is not an allowed character")
                    solve = False
                    # join the characters back into one equation
                elif i == '=':
                    # determines if equals sign is present in every line
                    containsoperator = True
        output_equations.append(tmp_eqn)
        if not containsoperator:
            print("There is a missing '=' sign or a lone variable in the equations list")
            solve = False
        return solve

    # this don't work

    # def check(self):
    #     solve = True
    #     for equation in self.equations:
    #         has_errors = EquationErrorCheck.EquationErrorCheck(equation).contains_errors()
    #         if has_errors is True:
    #             solve = False
    #     return solve

    def parse_eqns_from_string(self, instring):
        stringequations = instring
        equations = stringequations.split("\n")
        equations2 = list(filter(None, equations))
        return equations2

    def format(self, equations):
        """
        the purpose of this function is to take the list of equations,
        - remove spaces 
        - uppercase all characters
        - change certain characters for processing
            ^ -> **
            
        example input
        ['x = y+ 1', 'y = 2^2', 'squ = 2', 'yo = 20']
        example output
        ['X=Y+1', 'Y=2**2', 'SQU=2', 'YO=20']
        *************************
        """
        output_equations = []
        for eqn in equations:
            eqn = eqn.replace(" ","") # remove spaces
            eqn = eqn.replace("^", "**") # for python to understand exponentials
            outeqn = eqn.upper()
            output_equations.append(outeqn)
        return output_equations

    def variables(self, equation=False):
        """
        split equations by the all known operators
        store in list for processing
        :param equation:
        :return:
            list of variables (without copies)
        """
        import re
        variables = []

        if equation is False:
            for one_equation in self.equations:
                # regular expression for splitting strings with given characters
                split_equations = re.split(r"[=+\-^*/\\()\[\]]", one_equation)
                for i in split_equations:
                    if self.isvariable(i) and i not in variables:
                        variables.append(i)
            return variables

        elif isinstance(equation, str):
            split_equations = re.split(r"[=+\-^*/\\()\[\]]", equation)
            for i in split_equations:
                if self.isvariable(i) and i not in variables:
                    variables.append(i)
            return variables

        else:
            raise TypeError

    def variables_in_eqn(self,equation):
        pass

    def variable_dict(self):
        """
        :rtype: dict
        :return:
            list of variables (without copies)
            variable dict = {equation:[variables], ..}
        """
        import re
        variables = []
        variable_dict = {}
        for each_equation in self.equations:
            # regular expression for splitting strings with given characters
            variable_dict[each_equation] = self.variables(each_equation)
        return variable_dict

    def isvariable(self, variable):
        if variable is '':
            return False
        elif variable.isnumeric():
            return False
        elif self.isfloat(variable):
            return False
        return True

    def isfloat(self, i):
        '''
        will determine if a string can be interpreted as a float
        :param i: string
        :return:
        '''
        try:
            float(i)
            return True
        except ValueError:
            return False

    def equation_dict(self):
        # split equations by the equals sign
        import re
        equation_dict = {}
        for i in self.equations:
            split_equations = re.split(r"[=]", i)
            equation_dict[i] = split_equations
        return equation_dict

    def debug(self):
        # print input equations
        output = ["\nEntered Equations: "]
        for i in self.entered_equations:
            output.append(i)

        # formatted equations and numbered
        debug_equations = self.format(self.equations)
        output.append("\nFormatted Equations: ")
        for i, item in enumerate(debug_equations):
            output.append(str(str(i+1) + " " + item))

        # check equations
        solve = self.check()
        # print variables input
        output.append("\nVariables")
        variables = self.variables()
        for i in variables:
            output.append(i)

        # Variable dictionary
        output.append("\nVariable Dictionary")
        for i in self.variable_dictionary:
            output.append(str(i))
            output.append(str(self.variable_dictionary[i]))

        # Equation dictionary
        output.append("\nEquation Dictionary")
        equation_dict = self.equation_dict()
        for i in equation_dict.items():
            output.append(str(i[0]))
            output.append(str(i[1]))

        # join all and print
        output = "\n".join(output)
        if solve:
            return print(output)
        else:
            return print("Error in equation checking")


if __name__ == "__main__":
    import readfile as rf
    input_file = "1eqn"
    eqns = rf.readfile(input_file)
    equations_object = EquationsClass(eqns)
    print("\nReading input file of name: " + input_file)
    equations_object.debug()
    print("\n")


from the_ends.EquationObject import EquationsClass
import math


class Solver(object):
    """
    docstring
    take in list of equations
    parse them
    solve them
    spit them back at the user in a rather violent but satifactory manner
    """

    def __init__(self, input_equations, debug=False):
        self.looplimit = 50
        self.eqn_obj = EquationsClass(input_equations)
        self.vdict = self.eqn_obj.variable_dict()
        self.debug = debug
        self.equations = input_equations

    def solve(self):
        #MAIN
        # this is the order in which the solver class is completed.
        eqn_block, solvable_vars = self.primary_parser
        exelist, resultlist = self.exportSortedEquations(eqn_block, solvable_vars)
        return exelist, resultlist


    @property
    def primary_parser(self):
        '''
        :param
        self.equations

        :return:
        '''

        eqn_block = [self.equations, []]
        solvable_vars = []
        parsed = False
        count = 0
        block_num = 0

        while not parsed:
            count += 1
            # *****************************************************************
            # Loop Termination Items
            # *****************************************************************
            # Loop is terminated if:
            # A: self.looplimit is reached
            # B: Current block has run out of equations
            # C: (NEED TO ADD) Remaining Block has multiple (self-contingent) unknowns
            if count > self.looplimit:
                print("Loop Limit reached on block solver")
                raise SolutionError

                # break if loop limit is reached
            # DO NOT CHANGE FROM == TO is
            elif eqn_block[block_num] == []:
                # when there are no more equations to sort,
                # terminate the loop
                eqn_block.remove([])
                # DO NOT MODIFY THIS LINE! things will break, frustration will ensue
                if eqn_block[block_num] == []:
                    eqn_block.remove([])
                if self.debug == 1:
                    print("--------------------------------------------"
                          "--------------------"
                          "\nFinal Equation Block: ")
                    for i in range(0, len(eqn_block)):
                        print("     " + str(eqn_block[i]))
                break

            # *****************************************************************
            # Working with Current Block
            # *****************************************************************

            # by default the block is not solvable
            blockissolvable = False

            # simplify the reference to the current block
            current_block = eqn_block[block_num]

            # collect variables in the current_block
            current_block_vars = EquationsClass(current_block).variables()

            # adds solvable variables to the current block variables
            block_vars = solvable_vars + current_block_vars

            # list(set()) operation to sort and remove duplicates
            block_vars = list(set(block_vars))
            block_vars = list(filter(None, block_vars))
            solvable_vars = list(set(solvable_vars))

            if self.debug is True:
                print("\nCurrent Block: " + str(current_block))
                print("Block Number: " + str(block_num + 1))
                # prints activity for debugging

            # Check if the block is already solvable
            # if variables in current block and
            #                 previous blocks
            #                 == solvable variables
            # --> revisit order of logic
            if set(block_vars) == set(solvable_vars):
                blockissolvable = True
                if self.debug == 1: print("block vars and solvable vars are the same")

            elif set(block_vars) != set(solvable_vars):
                tmp_solvable_vars = []
                next_block_equation_list = []
                number_of_unsolvable_equations = 0
                for current_equation in current_block:
                    # *****************************************************************
                    # Working with Current Equation (sorting single unknowns)
                    # *****************************************************************
                    # 1. Determine if solvable
                    #       By comparing solvable vars to current equation
                    # 2. If not solvable
                    #       Push Equation to next Block
                    # 3. If solvable
                    #       Add variables to Solvable Variables
                    #       Leave Equation in current block

                    solvable = self.issolvable(current_equation, solvable_vars, False)
                    if solvable is False:
                        number_of_unsolvable_equations +=1
                        # move equation to next block
                        next_block_equation_list.append(current_equation)

                        if self.debug == 1: print("Not Solvable--> " + str(current_equation))
                    elif solvable is True:
                        # add variables to solvable variables
                        tmp_eqn_vars = self.vdict[current_equation]
                        for v in tmp_eqn_vars:
                            tmp_solvable_vars.append(v)
                        if self.debug == 1: print("Solvable        " + str(current_equation))
                        # print("variables in equation test" + str(tmp_eqn_vars))
                if len(current_block) == number_of_unsolvable_equations:
                    raise SolutionError(current_block)

                # what does this do
                for current_equation in next_block_equation_list:
                    eqn_block[block_num + 1].append(current_equation)
                    eqn_block[block_num].remove(current_equation)

                # append tmp_solvable_vars to solvable vars
                solvable_vars.extend(tmp_solvable_vars)
                # for variable in tmp_solvable_vars:
                #    solvable_vars.append(variable)
                if self.debug == 1: print("block vars and solvable vars are not the same")
                # iterate through current block equations
                # determine if not solvable

            if self.debug == 1:
                print("Block Variables \n     " + str(block_vars))
                print("Solvable Variables \n     " + str(solvable_vars))
                print("current equation block: ")
                for i in range(0, len(eqn_block)):
                    print("     " + str(eqn_block[i]))
                print("Is block solvable: " + str(blockissolvable))

            if blockissolvable is True:
                # If the block is solvable, advance to next block
                block_num += 1
                eqn_block.append([])
                if self.debug == 1: print("next block\n--------------------------"
                                          "--------------------------------------")
        return eqn_block, solvable_vars

    def exportSortedEquations(self, eqn_block, solvable_vars):
        # *****************************************************************
        # Export of solvable list of variables
        # *****************************************************************
        all_vars = sorted(set(solvable_vars))
        # executable list to be run by python interpreter
        execute_list = []
        results_list = []
        for current_block in eqn_block:
            for current_equation in current_block:
                execute_list.append(current_equation)
        for current_variable in all_vars:
            results_list.append(current_variable)
        return execute_list, results_list

    def solver_selection(self):
        """
        Will read in execute_list and apply additional solvers as nessesary?
        - python math
        - fsolve
        (solving actually happens in tmp.py (might have changed))
        :parameter:
        solve("equation", "for this variable", "with this guess")
        :return:
        float of answer
        error message?
        """
        pass

    def issolvable(self, equation, solvablevars, debug):
        """
        takes in a single equation and variables to compare it against
        determines variables in the equation
        returns a True/False condition
        :param equation: 
            str single equation 'X2=X1+1'
        :param solvablevars:
            list of already solvable variables
        :param debug:
            set True for debug of this function
        :return solvable: 
            True if the equation, given the solvable variables
            is able to be solved
        """

        solvable = False
        svariables = self.vdict[equation]
        common_variables = list(set(svariables).intersection(solvablevars))
        if debug:
            print(svariables)
            print(solvablevars)
            print(common_variables)
        for i in common_variables:
            svariables.remove(str(i))
        if len(svariables) <= 1:
            solvable = True
        return solvable


class Error(Exception):
    """Base class for exceptions in this module."""
    pass


class SolutionError(Error):
    """Exception raised for errors in the input.

    Attributes:
        expression -- input expression in which the error occurred
        message -- explanation of the error
    """

    def __init__(self, expression):
        self.expression = expression
        self.message = "These equations are unsolvable: \n" + \
                       str(self.expression)


class KevinSolver1:
    '''
    NOT CURRENTLY WORKING
    WORKING IN EQUATION SOLVER RESIDUALS
    say this is block 1
    x*2 = 5+y
    y+2 = x-2
    splits to
    x, x*2
    y, 5+y
    y, y+2
    x, x-2
    creates equation going to

    '''
    def __init__(self):
        pass

    def stringtofunction(self, equationstring="6*x"):
        mydict = {'eq': equationstring}
        A = mydict['eq']
        return A

    def fn1_1(self, x):
        return 2 * x

    def fn1_2(self, y):
        return y + 5

    def fn2_1(self, y):
        return y+2

    def fn2_2(self, x):
        return x-2

    def fn1_solve(self):
        pass

    def fn2_solve(self):
        pass

    def firsttry(self):

        x=1.0
        y=-3.0
        var1 = "x"
        var2 = "y"
        tries = 4
        maxerr = .00001
        delta = .001
        for tries in range(tries):
            ans1 = (self.solve2(self.fn1_1, self.fn1_2, x, y, var1, var2))
            ans2 = (self.solve2(self.fn2_1, self.fn2_2, y, x, var2, var1))
            ans1_delta = (self.solve2(self.fn1_1, self.fn1_2, x+delta, y+delta, var1, var2))
            ans2_delta = (self.solve2(self.fn2_1, self.fn2_2, y+delta, x+delta, var2, var1))
            #print(ans1, ans2)
            print("--------------")
            print(var1, ans1[var1], ans2[var1])
            print(var2, ans1[var2], ans2[var2])
            err1 = ans1[var1]-ans2[var1]
            err2 = ans1[var2]-ans2[var2]
            err = err1*err2
            print("x error:", err1,"y error:", err2)
            #x=(ans1[var1]+ans2[var1])/2.0
            #y=(ans1[var2]+ans2[var2])/2.0
            slope1_1 = (ans1_delta[var1]-ans1[var1])/delta # add to ans dict
            slope1_2 = (ans2_delta[var1]-ans2[var1])/delta
            slope2_1 = (ans1_delta[var2]-ans1[var2])/delta  # add to ans dict
            slope2_2 = (ans2_delta[var2]-ans2[var2])/delta
            if slope1_2 == 0:
                slope1_2 = 1.0
            if slope2_2 == 0:
                slope2_2 = 1.0
            x -= err/(slope1_1*slope1_2)
            y -= err/(slope2_1*slope2_2)
            print("New X:", x, "New Y:", y)
        return

    def dx(self, fn, x, delta=0.001):
        return (fn(x + delta) - fn(x)) / delta

    def solve_old(self, fn, value, x=0.5, maxtries=1000, maxerr=0.00001):
        for tries in range(maxtries):
            err = fn(x) - value
            if abs(err) < maxerr:
                x = math.ceil(x * 100000) / 100000
                return x
            slope = self.dx(fn, x)
            x -= err / slope
        raise ValueError('no solution found')

    def solve2(self, fn1, fn2, x1, x2, x1str, x2str, maxtries=1000, maxerr=0.00001):
        # currently finds a solution to an indeterminate set of equations
        err = 1.0
        for tries in range(maxtries):
            err = fn1(x1) - fn2(x2)
            if abs(err) < maxerr:
                x1 = math.ceil(x1 * 100000) / 100000
                x2 = math.ceil(x2 * 100000) / 100000
                return {x1str: x1, x2str: x2}
            slope1 = self.dx(fn1, x1)
            x1 -= err / slope1
            slope2 = self.dx(fn2, x2)
            x2 -= err / slope2
        if maxtries == 1:
            return {x1str: x1, x2str: x2}
        else:
            raise ValueError('no solution found')



def testsolverclass():
    """
        Testing Solver Class------------------------
        """
    # test of is solvable
    try:
        import Eqn_solver.readfile as rf

        input_file = "1eqn"
        eqns = rf.readfile(input_file)
        equations_object = EquationsClass(eqns)
        print("Reading input file of name: " + input_file)
        equations = equations_object.equations
        variables = equations_object.variables()
        # print(equations)
        # print(variables)
        # print(Solver(equations).issolvable(equations[0], variables[2], 0))
        exelist, resultslist = Solver(equations, debug=True).solve()
        peqns = EquationsClass(eqns).equations
        print("exelist:")
        print(exelist)
        print("results list:")
        print(resultslist)
        print("equations from equations class:")
        print(peqns)
        print("--------------\nTESTING\n--------------")
        for i in exelist:
            print(i)
            exec(i)
        print("\n")
        for i in resultslist:
            print(i, "=", float(eval(i))) # implement this elsewhere!!!!
            # solve_and_print_results(peqns, exelist)
    except SolutionError:
        print("\n\nTHE SOLVER BROKE!\nOH NO!!\n"
              "maybe if you try again you will get the same result...")


def testkevinsolver1class():
    solver = KevinSolver1()
    solver.firsttry()
    return


if __name__ == "__main__":
    testsolverclass()
    #testkevinsolver1class()



import re
from the_ends.EquationObject import EquationsClass


class EquationErrorCheck:
    '''
    This class if to check for errors in individual equations
    it is intended to interface with the gui
    the error message variable will be used for interactive
    feedback when coding in the equation solver
    
    '''
    def __init__(self, equation, line_number=0):
        self.equation = equation
        self.line_number = line_number
        self.variable_dict = EquationsClass(equation).variable_dictionary
        # error message that will be displayed line by line
        # includes active messages
        self.errormessage = ''
        self.leftequation = ''
        self.rightequation = ''
        self.contains_errors = True
        self.checkline()

    def checkline(self):
        """
        1. check for correct number of equals
        2. check for trailing symbols
        3. check for symbols not allowed
        :return:
        """
        # assumed to contain errors until proven false
        contains_errors = True
        while contains_errors is True:

            # returns true if there is only one equals sign
            check_equals = self.check_for_equals()
            if check_equals is False:
                # check for correct number of equals
                self.errormessage = "This equation must contain only one = "
                break
            # splits the equation into a left and right side
            # Split equation on the equals using
            # self.leftequation
            # self.rightequation
            elif check_equals is True:

                self.split_equation()

            # uses left and right equations
            # checks if there is a trailing opereator in both
            # returns True if there are no errors
            # returns False if an error is found
            check_trailing = self.check_trailing_operators()
            if check_trailing is False:
                break

            # checks for common symbols not allowed by the program
            check_symbols = self.check_symbols()
            if check_symbols is False:
                break

            # if no errors are found it will make it to here
            self.errormessage = "no errors"
            contains_errors = False

        # sets the contains errors variable for the class
        self.contains_errors = contains_errors
        return self.errormessage

    def check_for_equals(self):
        # need to find all = in line
        # only passes if there's one and only one
        answer = re.findall(r"[=]", self.equation)
        if len(answer) is 1:
            correct_equals = True
        else:
            correct_equals = False
        return correct_equals

    def split_equation(self):
        # split equation by = first
        self.leftequation, self.rightequation = re.split(r"[=]", self.equation)

    def check_trailing_operators(self):
        """
        Checks front and back of each subequation(left and right of equals)
        for extraneous operators
        :return:
        """
        # r"[=+\-^*/\\()\[\]]"
        # does this pass the check?
        trailing_operators_pass = True
        for eqstring in [self.leftequation, self.rightequation]:
            lineend = re.search(r"\+$|-$|\*$|\^$|\($|/$", eqstring)
            linestart = re.search(r"^\+|^-|^\*|^\^|^\)|^/", eqstring)
            if lineend:
                end = lineend.group(0)
                self.errormessage = \
                    "There is a trailing " + str(end) + \
                    " in this equation"
            elif linestart:
                start = linestart.group(0)
                self.errormessage = \
                    "There is a trailing " + str(start) + \
                    " in this equation"
            # print(bool(linestart))
            # print(bool(lineend))
            if bool(linestart) or bool(lineend):
                trailing_operators_pass = False
        return trailing_operators_pass

    def check_symbols(self):
        symbols_not_allowed = ['?', '@', '&', '`', '~', '#', '!', '$', '%']
        for i in self.equation:
            if i in symbols_not_allowed:
                self.errormessage = str(i) + " is not an allowed symbol"
                return False

        return True

    def debug(self):
        pass


if __name__ == '__main__':
    EQ = EquationErrorCheck("x=4+4+a")
    print(EQ.variable_dict)
    print("\n>>>Checkline results:")
    print(EQ.checkline())
    print("\n>>>Contains Errors? \n", EQ.contains_errors)
    # EQ.check_trailing_operators()
    # print(">>> Does the equation have the correct number of equals?")
    # print(EQ.check_for_equals())
    # print(">>> Does the equation have no trailing operators?")
    # print(EQ.check_trailing_operators())
    # print(EQ.errormessage)



#!/usr/bin/python3

def readfile(filename):
    '''
    :param 
    filename: 
        input is a text file for input
    :return: 
    Equations: 
        lines with potential equations on them
    num_line:
        number of equation lines
    '''
    # open file
    with open(filename, mode='r') as f:
        line_list = []
        # read in lines
        while True:
            line = f.readline()
            if len(line) == 0:
                break
            line_list.append(line)
    outstring = "".join(list(filter(None, line_list)))
    return outstring


def readstring(input_string):
    '''
    :param 
    filename: 
        input is a text file for input
    :return: 
    Equations: 
        lines with potential equations on them
    num_line:
        number of equation lines
    '''

    num_line = 0
    # read in lines
    equations = []
    line_list = input_string.split("\n")
    for line in line_list:
        num_line += 1
        # go through each line in equations
        # and append non-empty lines
        equations.append(line)
    equations = list(filter(None, equations))
    return [equations, num_line]

if __name__ == "__main__":
    print(readfile("1eqn"))
    input_str = 'x4 = a + b+c+d+f+x3^2\na = 2\nb = 5\n' \
                'c = 4\ne = 4+f\nf = b\ng = b\nd = c + e\n' \
                'x1 = 20\nx2 = x1+3\nx3 = x2 -10\n'
    print(readstring(input_str))



#!/usr/bin/python

# setup.py info things
from setuptools import setup, find_packages


# Distributing commands:
# python setup.py sdist (builds the project from this file)
# twine upload dist/* (uploads everything in that folder to pypi)
setup(
    name="the_ends",
    packages = ['the_ends'],
    version="0.1.9",
    author="Kevin Cavender",
    description='Implicit Equation Solver (will reorder equations for maths)',
    author_email='kac1200@gmail.com',
    url="https://github.com/Kevincavender/the_ends",
    download_url='https://github.com/Kevincavender/the_ends/archive/0.1.tar.gz',
    keywords = ['equation', 'engineer', 'math', 'calculator'],
    entry_points={
        'console_scripts':[
            'solve=the_ends.__main__:main'
        ]
    }
  )
